searchData={"items":[{"type":"module","title":"Saxaboom","doc":"The main entrypoint for parsing a document from a string or Stream.","ref":"Saxaboom.html"},{"type":"function","title":"Saxaboom.parse/3","doc":"Parses a given document into a user-defined struct (see: [Saxaboom.Mapper](https://hexdocs.pm/saxaboom/Saxaboom.Mapper.html) for more information)\n\nTakes as arguments:\n  - `xml`: a string or stream of XML data\n  - `into` a `Saxaboom.Mapper` type that describes the document to be parsed\n\nTakes as keyword arguments:\n  - `adapter`: an atom describing the adapter to use when parsing the document\n     - Defaults to `xmerl`\n  - `parser_options`: a list of options to pass down to the chosen adapter. Refer to the parser documentation for more information.\n\nReturns:\n  - A tuple containing the status and the parsed structure, optionally error information surfaced from the underlying parser","ref":"Saxaboom.html#parse/3"},{"type":"module","title":"Saxaboom.Element","doc":"Describes an element with attributes as they are surfaced from a given parsing adapter.","ref":"Saxaboom.Element.html"},{"type":"protocol","title":"Saxaboom.ElementCollectable","doc":"","ref":"Saxaboom.ElementCollectable.html"},{"type":"function","title":"Saxaboom.ElementCollectable.cast_attributes/2","doc":"","ref":"Saxaboom.ElementCollectable.html#cast_attributes/2"},{"type":"function","title":"Saxaboom.ElementCollectable.cast_characters/3","doc":"","ref":"Saxaboom.ElementCollectable.html#cast_characters/3"},{"type":"function","title":"Saxaboom.ElementCollectable.cast_nested/3","doc":"","ref":"Saxaboom.ElementCollectable.html#cast_nested/3"},{"type":"function","title":"Saxaboom.ElementCollectable.element_definition/2","doc":"","ref":"Saxaboom.ElementCollectable.html#element_definition/2"},{"type":"type","title":"Saxaboom.ElementCollectable.t/0","doc":"All the types that implement this protocol.","ref":"Saxaboom.ElementCollectable.html#t:t/0"},{"type":"module","title":"Saxaboom.Mapper","doc":"The main user interface for defining mapping definitions.\n\n`Saxaboom.Mapper` exposes a micro-DSL for describing the expected structure of a given document. There are three\ncomponents to the `Saxaboom.Mapper` interface:\n\n- `document`: a directive describing the \"envelope\" of the incoming document (read: the whole document). This exposes the internal DSL.\n- `element`: a directive to match zero to one element and collect into a single struct field\n- `elements`: a directive to match zero to N elements and collect in a list in the order they are encountered","ref":"Saxaboom.Mapper.html"},{"type":"macro","title":"Saxaboom.Mapper.document/1","doc":"Begins the definition of a \"document\", that is, a section of parseable content.","ref":"Saxaboom.Mapper.html#document/1"},{"type":"macro","title":"Saxaboom.Mapper.element/2","doc":"Defines a structure field that can match against 0 or 1 elements in the given document.\n\nArguments:\n  - `name` is the name of the tag to match against, case sensitive\n\nOptions:\n  - `:as` provides the name to be used on the struct when parsing, defaults to the name of the tag\n  - `:value` identifies the property to extract from the tag. Can be an attribute name, defaults to the text content of the node\n  - `:with` is a keyword list of attributes and expected attribute values. The `:with` must perfectly match against a subset of the node attributes\n  - `:cast` is a symbol or a user-defined function to transform the extracted `:value`, see `Saxaboom.Utils.Caster` for more details.\n  - `:into` is another mapper or type that implements the `Saxaboom.ElementCollectable` protocol. Child nodes will be parsed into this structure until the parser has encountered the closing tag of the node that began the `:into`.\n  - `:default` is the default value of the node, if no tags are parsed. `nil` by default","ref":"Saxaboom.Mapper.html#element/2"},{"type":"macro","title":"Saxaboom.Mapper.elements/2","doc":"Defines a structure field that can match against 0 or N elements in the given document. The list will be parsed and extracted\nin document order.\nDefaults to an empty list.\n\nArguments:\n  - `name` is the name of the tag to match against, case sensitive\n\nOptions:\n  - `:as` provides the name to be used on the struct when parsing, defaults to the name of the tag\n  - `:value` identifies the property to extract from the tag. Can be an attribute name, defaults to the text content of the node\n  - `:with` is a keyword list of attributes and expected attribute values. The `:with` must perfectly match against a subset of the node attributes\n  - `:cast` is a symbol or a user-defined function to transform the extracted `:value`, see `Saxaboom.Utils.Caster` for more details.\n  - `:into` is another mapper or type that implements the `Saxaboom.ElementCollectable` protocol. Child nodes will be parsed into this structure until the parser has encountered the closing tag of the node that began the `:into`.","ref":"Saxaboom.Mapper.html#elements/2"},{"type":"module","title":"Saxaboom.Utils.Caster","doc":"Defines semantics for standard casting mechanisms to be called after a given value has been parsed from a document.\nThe current out of the box casters include:\n\n- `:noop` <-- the default\n- `:string`\n- `:float`\n- `:integer`\n- `:atom`\n- `:existing_atom`\n- `:boolean`\n  - If the input is a string, the conversion will consider `[\"1\", \"on\", \"true\", \"yes\", \"y\", \"t\"]` as truthy values and all other values as falsy\n  - If the input is not a string, the conversion will force a boolean value by calling `!!` on the provided value\n\nIf the above casters do not cover a particular case (for example, Date parsing) then a function can be provided instead\nto resolve the correct value.","ref":"Saxaboom.Utils.Caster.html"},{"type":"function","title":"Saxaboom.Utils.Caster.cast_value/2","doc":"","ref":"Saxaboom.Utils.Caster.html#cast_value/2"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [Unreleased]\n\n\n## [0.1.0] - 2023-10-04\n\n- Initial release\n\n\n[0.1.0]: https://github.com/ducharmemp/saxaboom/commits/v0.1.0","ref":"changelog.html"},{"type":"extras","title":"License","doc":"MIT License\n\nCopyright (c) 2023 Matthew DuCharme\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"},{"type":"extras","title":"Saxaboom","doc":"# Saxaboom\n\n[![Test suite](https://github.com/ducharmemp/saxaboom/actions/workflows/elixir.yml/badge.svg?branch=main)](https://github.com/ducharmemp/saxaboom/actions/workflows/elixir.yml?branch=main)\n[![Module Version](https://img.shields.io/hexpm/v/saxaboom.svg)](https://hex.pm/packages/saxaboom)","ref":"readme.html"},{"type":"extras","title":"Installation - Saxaboom","doc":"Add `saxaboom` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:saxaboom, \"~> 0.1.0\"}\n  ]\nend\n```\n\nOptionally add an XML parsing libray based on your needs, view the benchmarks if you'd like to see a head-to-head comparison\nbetween XML SAX parsers. If a third-party XML library is not desired, [xmerl](https://www.erldocs.com/18.0/xmerl/xmerl_sax_parser.html?i=652) will be used.\n\nSaxaboom currently has adapters for:\n- [Saxy](https://github.com/qcam/saxy)\n- [Erlsom](https://github.com/willemdj/erlsom)\n- [xmerl_sax_parser](https://www.erldocs.com/18.0/xmerl/xmerl_sax_parser.html?i=652)\n\nIf using Saxy, for example, your `mix.exs` would look something like:\n\n```elixir\ndef deps do\n  [\n    {:saxaboom, \"~> 0.1.0\"},\n    {:saxy, \"~> ...\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"About - Saxaboom","doc":"Saxaboom is a data-mapper capable of transforming XML via SAX into a tree of structs. Think\nof it as xpath selectors but with the ability to work on potentially infinite streams of data with low memory usage since it only\nworks with a few elements at a time relative to the rest of the document. If you're familiar with https://github.com/pauldix/sax-machine\nthen this library should more or less look exactly the same with some minor tweaks.","ref":"readme.html#about"},{"type":"extras","title":"Example - Saxaboom","doc":"You can run the code for yourself in `examples/bookstore` via `mix run -e \"Bookstore.list_books('books.xml')\"`\n\nDefine a mapping layout of some kind using `Saxaboom.Mapper`:\n\n```elixir\ndefmodule Bookstore.Book do\n  use Saxaboom.Mapper\n\n  document do\n    element :book, as: :id, value: :id\n    element :author\n    element :title\n    element :genre\n    element :price, cast: :float\n    element :publish_date, cast: &__MODULE__.parse_date/1\n    element :description\n  end\n\n  def parse_date(value), do: Date.from_iso8601(value)\nend\n\ndefmodule Bookstore.Catalog do\n  use Saxaboom.Mapper\n\n  document do\n    elements :book, as: :books, into: %Book{}\n  end\nend\n```\n\nAnd the resulting (trunacated) output:\n```\n{:ok,\n %Bookstore.Catalog{\n   books: [\n     %Bookstore.Book{\n       id: \"bk101\",\n       author: \"Gambardella, Matthew\",\n       title: \"XML Developer's Guide\",\n       genre: \"Computer\",\n       price: 44.95,\n       publish_date: {:ok, ~D[2000-10-01]},\n       description: \"An in-depth look at creating applications\\n      with XML.\"\n     },\n     %Bookstore.Book{\n       id: \"bk102\",\n       author: \"Ralls, Kim\",\n       title: \"Midnight Rain\",\n       genre: \"Fantasy\",\n       price: 5.95,\n       publish_date: {:ok, ~D[2000-12-16]},\n       description: \"A former architect battles corporate zombies,\\n      an evil sorceress, and her own childhood to become queen\\n      of the world.\"\n     },\n     ...\n   ]\n }}\n```\n\nAs you can see the overall structure looks good but there seems to be some extra whitespace present in the `description`. To begin playing around with the\nlibrary, a good place to start would be to modify the `Book` structure to remove said whitespace so the first entry simply reads:\n\n```\n%Bookstore.Book{\n  id: \"bk101\",\n  author: \"Gambardella, Matthew\",\n  title: \"XML Developer's Guide\",\n  genre: \"Computer\",\n  price: 44.95,\n  publish_date: {:ok, ~D[2000-10-01]},\n  description: \"An in-depth look at creating applications with XML.\"\n},\n```","ref":"readme.html#example"},{"type":"extras","title":"Caveats and Security - Saxaboom","doc":"#","ref":"readme.html#caveats-and-security"},{"type":"extras","title":"On security for XML - Saxaboom","doc":"Parsing untrusted input from external sources is always fraught with peril! Because there are a myriad of ways that attackers\ncan exploit XML parsers, Saxaboom allows the user to pass additional arguments directly to the underlying parsing library to control\nthings like DTD validation/expansion. Please refer to the XML library of choice for their recommendations/special considerations when\nparsing untrusted input.\n\n#","ref":"readme.html#on-security-for-xml"},{"type":"extras","title":"On parsing semantics - Saxaboom","doc":"##","ref":"readme.html#on-parsing-semantics"},{"type":"extras","title":"How Saxaboom works - Saxaboom","doc":"Saxaboom maintains a few stacks to maintain state while walking down the tree, a stack of elements that have been visited but not closed, and a stack of `Mapper` that can receive data. If an `into` field is encountered, Saxaboom pushes the `into` type onto the handler stack. If an element closing tag is encountered, Saxaboom sends the event to the handler at the top of the handler stack. Saxaboom continues this pushing and popping of `Mapper` and elements until the end of the document has been reached.\n\n##","ref":"readme.html#how-saxaboom-works"},{"type":"extras","title":"Semantics - Saxaboom","doc":"- `element`: Last element in the current parsing tree wins\n- `elements`: All elements are collected in document order.\n- `element` and `elements` do not care about the depth of a given item within the tree. If they encounter a tag that matches something in the current `Mapper`, it'll be parsed into the type. This results in an auto-flattening of the tree, which is desirable in a lot of cases. If this is not desirable, consider defining a few nested `Mapper` types.\n- `into` types receive the tag that starts the nested section\n  - This is why we could extract the `id` from the `book` tag within the `Book` struct in the example above\n- The default behavior of both the `element` and `elements` types is to extract the characters present between the opening and closing tags if a `value` definition is not present\n- The default behavior is to leave the value extracted from a tag as-is unless otherwise specified by a `cast` function.","ref":"readme.html#semantics"},{"type":"extras","title":"Benchmarks - Saxaboom","doc":"Tests were run using `MIX_ENV=prod mix run bench.exs` in the `bench` directory.","ref":"readme.html#benchmarks"}],"content_type":"text/markdown"}